/*
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the Network Research
 * 	Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /usr/src/mash/repository/mash/mash-1/codec/module.h,v 1.8 1999/01/18 22:25:55 weitsang Exp $ (LBL)
 */

#ifndef mash_module_h
#define mash_module_h

#include "tclcl.h"
#include "config.h"
#include "rtp.h"

class RTP_BufferPool;
class pktbuf;

class Buffer {
public:
	virtual Buffer* copy() = 0;
	virtual void release();
};

struct AudioFrame : public Buffer {
public:
	inline Buffer* copy() { return 0; }//currently not used
	inline AudioFrame(u_int32_t ts, const u_int8_t* bp, int len) :
		ts_(ts), bp_(bp), len_(len) {}
	u_int32_t ts_;
	const u_int8_t* bp_;
	int len_;
};

class VideoFrame : public Buffer {
    public:
	inline Buffer* copy() { return 0; }//currently not used
	inline VideoFrame(u_int32_t ts, u_int8_t* bp, int w, int h,
			  int layer) :
		ts_(ts), bp_(bp), width_(w), height_(h), layer_(layer) { }
	u_int32_t ts_;
	u_int8_t* bp_;
	int width_;
	int height_;
	int layer_;
};

class YuvFrame : public VideoFrame {
    public:
	inline YuvFrame(u_int32_t ts, u_int8_t* bp, u_int8_t* crvec,
			int w, int h, int layer = 0) :
		VideoFrame(ts, bp, w, h, layer), crvec_(crvec) {}
	const u_int8_t* crvec_;
};

class JpegFrame : public VideoFrame {
    public:
	inline JpegFrame(u_int32_t ts, u_int8_t* bp, int len, int q, int type,
			int w, int h) :
		VideoFrame(ts, bp, w, h, 0), len_(len), q_(q), type_(type) {}
	int len_;
	int q_;
	int type_;
};

class DCTFrame : public VideoFrame {
    public:
	inline DCTFrame(u_int32_t ts, short* bp, u_int8_t* crv,
		int w, int h, int q = -1) :
		VideoFrame(ts, (u_int8_t*)bp, w, h, 0), crvec_(crv), q_(q) {}

	const u_int8_t *crvec_;
	int q_;			// original q (if applicable)
};

class Module : public TclObject {
public:
	Module* next;
	virtual void recv(Buffer*) = 0;
	virtual void sched_exec (Buffer *buf) { recv(buf); }
	int command(int argc, const char*const* argv);
	inline Module* target() { return (target_); }
protected:
	Module();
	Module* target_;
};

class SourceModule : public Module {
public:
	virtual void recv(Buffer*) {}
};

class FrameModule : public Module {
protected:
	FrameModule();
	inline void size(int w, int h) {
		width_ = w;
		height_ = h;
		framesize_ = w * h;
	}
	inline int samesize(const VideoFrame* vf) {
		return (vf->width_ == width_ && vf->height_ == height_);
	}
	int width_;
	int height_;
	int framesize_;
};

class PacketModule : public Module {
public:
	virtual void recv(Buffer* p) {
		recv((pktbuf*)p);
	}
	virtual void recv(pktbuf* pb) = 0;
};

class EncoderModule : public FrameModule {
public:
	virtual int command(int argc, const char*const* argv);
	inline u_int32_t nb() const { return nb_; }
protected:
	EncoderModule();
	RTP_BufferPool* pool_;
	int mtu_;
	u_int32_t nb_;
};

class MultiInputManager {
public:
	virtual void recv_data(pktbuf*) = 0;
	virtual void recv_ctrl(pktbuf*) = 0;
};

class DataInputHandler : public PacketModule {
public:
	DataInputHandler() : manager_(0) {}
	inline void manager(MultiInputManager* m) { manager_ = m; }
	virtual void recv(pktbuf* pb) {
		manager_->recv_data(pb);
	}
protected:
	MultiInputManager* manager_;
};

class ControlInputHandler : public DataInputHandler {
public:
	virtual void recv(pktbuf* pb) {
		manager_->recv_ctrl(pb);
	}
};

#endif
