/*
 * Copyright (c) 1994-1995 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and the Network Research Group at
 *      Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
static const char rcsid[] =
    "@(#) $Header: /usr/src/mash/repository/mash/mash-1/codec/video/encoder-h261.cc,v 1.2 1997/08/09 03:39:43 tecklee Exp $ (LBL)";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/time.h>
#include "inet.h"
#include "net.h"
#include "rtp.h"
#include "dct.h"
#include "p64/p64-huff.h"
#include "bsd-endian.h"
#include "tclcl.h"
#include "crdef.h"
#include "encoder-h261.h"
#include "transmitter.h"

#define HDRSIZE (sizeof(rtphdr) + 4)

static class H261EncoderMatcher : public Matcher {
public:
	H261EncoderMatcher() : Matcher("encoder") {}
	TclObject* match(const char* fmt) {
		if (strcasecmp(fmt, "h261/pixel") == 0)
			return (new H261PixelEncoder);
		if (strcasecmp(fmt, "h261/dct") == 0)
			return (new H261DCTEncoder);
		return (0);
	}
} h261_encoder_matcher;

/*
 * dead-zone threshold factors for luminance & chrominance quantization
 * tables for perceptual weighting.
 */
static const u_char scale0[64] = {
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
};
static const u_char lscale1[64] = {
	0, 0, 0, 0, 0, 1, 1, 1,
	0, 0, 0, 0, 0, 1, 1, 1,
	0, 0, 0, 0, 1, 1, 1, 1,
	0, 0, 0, 0, 1, 1, 1, 1,
	0, 0, 0, 1, 1, 1, 1, 1,
	0, 0, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1,
};
static const u_char cscale1[64] = {
	0, 0, 0, 1, 2, 2, 2, 2,
	0, 0, 0, 2, 2, 2, 2, 2,
	0, 0, 1, 2, 2, 2, 2, 2,
	1, 1, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2,
};

short bwdctblk[64];

H261Encoder::H261Encoder()
	    : bs_(0), ngob_(12), bb_(0), nbb_(0), width_(0), height_(0), 
	      framesize_(0)
{
	for (int q = 0; q < 32; ++q) {
		llm_[q] = 0;
		clm_[q] = 0;
	}
	bwdctblk[0] = 1024;
}

H261PixelEncoder::H261PixelEncoder()
{
	quant_required_ = 0;
	setq(10);
}

H261DCTEncoder::H261DCTEncoder()
{
	quant_required_ = 1;
	setq(10);
}

/*
 * Set up the forward DCT quantization table for
 * INTRA mode operation.
 */
void
H261Encoder::setquantizers(int lq, int mq, int hq)
{
	int qt[64];
	if (lq > 31)
		lq = 31;
	if (lq <= 0)
		lq = 1;
	lq_ = lq;

	if (mq > 31)
		mq = 31;
	if (mq <= 0)
		mq = 1;
	mq_ = mq;

	if (hq > 31)
		hq = 31;
	if (hq <= 0)
		hq = 1;
	hq_ = hq;

	/*
	 * quant_required_ indicates quantization is not folded
	 * into fdct [because fdct is not performed]
	 */
	if (quant_required_ == 0) {
		/*
		 * Set the DC quantizer to 1, since we want to do this
		 * coefficient differently (i.e., the DC is rounded while
		 * the AC terms are truncated).
		 */
		qt[0] = 1;
		int i;
		for (i = 1; i < 64; ++i)
			qt[i] = lq_ << 1;
		fdct_fold_q(qt, lqt_);

		qt[0] = 1;
		for (i = 1; i < 64; ++i)
			qt[i] = mq_ << 1;
		fdct_fold_q(qt, mqt_);

		qt[0] = 1;
		for (i = 1; i < 64; ++i)
			qt[i] = hq_ << 1;
		fdct_fold_q(qt, hqt_);
	}
}

void
H261Encoder::setq(int q)
{
	setquantizers(q, q / 2, 1);
}

#define MBPERGOB 33

void H261Encoder::size(int w, int h, int decimation)
{
	if (w == 352 && h == 288) {
		/* CIF */
		cif_ = 1;
		ngob_ = 12;
		bstride_ = 11;
		lstride_ = 16 * 352 - 352 / 2;
		cstride_ = 8 * 176 - 176 / 2;
	} else if (w == 176 && h == 144) {
		/* QCIF */
		cif_ = 0;
		ngob_ = 6; /* not really number of GOBs, just loop limit */
		bstride_ = 0;
		lstride_ = 16 * 176 - 176;
		cstride_ = 8 * 88 - 88;
	} else {
		/*XXX*/
		fprintf(stderr, "H261Encoder: H.261 bad geometry: %dx%d\n",
			w, h);
		exit(1);
	}
	int blkno = 0;
	decimation_ = decimation;
	int step = (decimation_ == 411)? 2 : 1;
	for (int gob = 0; gob < ngob_; gob += 2) {
		blkno_[gob] = blkno;
		uvblkno_[gob] = blkno >> step;
		/* width of a GOB (these aren't ref'd in QCIF case) */
		blkno_[gob + 1] = blkno + 11 * 2;
		uvblkno_[gob + 1] = (blkno >> step) + 11;

		/* advance to next GOB row */
		blkno += (MBPERGOB * 4) << cif_;
	}
	framesize_ = w * h;
	width_ = w;
	height_ = h;
}

/*XXX use 64-bits if available */
#define NBIT 32

#if BYTE_ORDER == LITTLE_ENDIAN
#define STORE_BITS(bb) \
	((u_char*)bs_)[(bc_ >> 3) + 0] = bb >> 24; \
	((u_char*)bs_)[(bc_ >> 3) + 1] = bb >> 16; \
	((u_char*)bs_)[(bc_ >> 3) + 2] = bb >> 8; \
	((u_char*)bs_)[(bc_ >> 3) + 3] = bb;
#else
#define STORE_BITS(bb) *(u_int*)(bs_ + (bc_ >> 3)) = (bb);
#endif

#define PUT_BITS(bits, n, nbb, bb) \
{ \
	nbb += (n); \
	if (nbb > NBIT)  { \
		int extra = (nbb) - NBIT; \
		bb |= (bits) >> extra; \
		STORE_BITS(bb) \
		bc_ += NBIT; \
		bb = (bits) << NBIT - extra; \
		nbb = extra; \
	} else \
		bb |= (bits) << NBIT - (nbb); \
}

/*
 * Make a map to go from a 12 bit dct value to an 8 bit quantized
 * 'level' number.  The 'map' includes both the quantizer (for the
 * dct encoder) and the perceptual filter 'threshhold' (for both
 * the pixel & dct encoders).  The first 4k of the map is for the
 * unfiltered coeff (the first 20 in zigzag order; roughly the
 * upper left quadrant) and the next 4k of the map are for the
 * filtered coef.
 */
char*
H261Encoder::make_level_map(int q, u_int fthresh)
{
	/* make the luminance map */
	char* lm = new char[0x2000];
	char* flm = lm + 0x1000;
	int i;
	lm[0] = 0;
	flm[0] = 0;
	q = quant_required_? q << 1 : 0;
	for (i = 1; i < 0x800; ++i) {
		int l = i;
		if (q)
			l /= q;
		lm[i] = l;
		lm[-i & 0xfff] = -l;

		if (l <= fthresh)
			l = 0;
		flm[i] = l;
		flm[-i & 0xfff] = -l;
	}
	return (lm);
}

/*
 * encode_blk:
 *	encode a block of DCT coef's
 */
void
H261Encoder::encode_blk(const short* blk, const char* lm)
{
	u_int bb = bb_;
	u_int nbb = nbb_;

	/*
	 * Quantize DC.  Round instead of truncate.
	 */
	int dc = (blk[0] + 4) >> 3;

	if (dc <= 0)
		/* shouldn't happen with CCIR 601 black (level 16) */
		dc = 1;
	else if (dc > 254)
		dc = 254;
	else if (dc == 128)
		/* per Table 6/H.261 */
		dc = 255;
	/* Code DC */
	PUT_BITS(dc, 8, nbb, bb);
	int run = 0;
	const int* colzag = &COLZAG[0];
	for (int zag; (zag = *++colzag) != 0; ) {
		if (colzag == &COLZAG[20])
			lm += 0x1000;
		int level = lm[((const u_short*)blk)[zag] & 0xfff];
		if (level != 0) {
			int val, nb0;
			huffent* he;
			if (u_int(level + 15) <= 30 &&
			    (nb0 = (he = &hte_tc[((level&0x1f) << 6)|run])->nb))
				/* we can use a VLC. */
				val = he->val;
			else {
				 /* Can't use a VLC.  Escape it. */
				val = (1 << 14) | (run << 8) | (level & 0xff);
				nb0 = 20;
			}
			PUT_BITS(val, nb0, nbb, bb);
			run = 0;
		} else
			++run;
	}
	/* EOB */
	PUT_BITS(2, 2, nbb, bb);

	bb_ = bb;
	nbb_ = nbb;
}


#define BLK_MINMAX(blk, cmin, cmax) \
{ \
	register const short* p = blk + 1; \
	for (register int j = 63; --j >= 0; ) { \
		register int v = *p++; \
		if (v < cmin) \
			cmin = v; \
		else if (v > cmax) \
			cmax = v; \
	} \
}


/*
 * H261PixelEncoder::encode_mb
 *	encode a macroblock given a set of input YUV pixels
 */
void H261PixelEncoder::encode_mb(int mba, const u_char* frm, int blkno, 
				 int uvblkno, int boff, int how)
{
	/*
	 * if the default quantizer is too small to handle the coef.
	 * dynamic range, spin through the blocks and see if any
	 * coef. would significantly overflow.
	 */
	float* qt;
	register int q;
	if (how == CR_MOTION) {
		q = lq_;
		qt = lqt_;
	} else if (how == CR_BG) {
		q = hq_;
		qt = hqt_; 
	} else {
		/* must be at age threshold */
		q = mq_;
		qt = mqt_; 
	}

	int line = (11 << cif_) * 2;
	int blkx = blkno % line;
	int blky = blkno / line;
	int loff = (blky << 3) * width_ + (blkx << 3);

	int coff, voff, uvstride;
	int fs = framesize_;
	if (decimation_ == 411) {
		coff = ((blky / 2) << 3) * (width_ / 2) + ((blkx / 2) << 3);
		voff = fs >> 2;
		uvstride = width_ / 2;
	} else {
		coff = (blky << 3) * (width_ / 2) + ((blkx / 2) << 3);
		voff = fs >> 1;
		/* will cause fdct to use decimated uv */
		uvstride = width_;
	}
	/*
	 * encode all 6 blocks of the macro block to find the largest
	 * coef (so we can pick a new quantizer if gquant doesn't have
	 * enough range).
	 */
	short blk[64 * 6];
	short* uvblk;
	register int stride = width_;
	/* luminance */
	const u_char* p = &frm[loff];
	fdct(p, stride, blk, qt);
	fdct(p + 8, stride, blk + 64, qt);
	fdct(p + 8 * stride, stride, blk + 128, qt);
	fdct(p + 8 * stride + 8, stride, blk + 192, qt);
	if (color_) {
		p = &frm[fs + coff];
		fdct(p, uvstride, blk + 256, qt);
		fdct(p + voff, uvstride, blk + 320, qt);
	} else {
		memset(blk + 256, 0x0, 128 * sizeof(short));
		blk[256] = 1024;
		blk[320] = 1024;
	}

	if (q < 8) {
		register int cmin = 0, cmax = 0;
		BLK_MINMAX(blk, cmin, cmax);
		BLK_MINMAX(blk + 64, cmin, cmax);
		BLK_MINMAX(blk + 128, cmin, cmax);
		BLK_MINMAX(blk + 192, cmin, cmax);
		BLK_MINMAX(blk + 256, cmin, cmax);
		BLK_MINMAX(blk + 320, cmin, cmax);
		if (cmax < -cmin)
			cmax = -cmin;
		if (cmax >= 128) {
			/* need to re-quantize */
			register int s;
			for (s = 1; cmax >= (128 << s); ++s)
				;
			q <<= s;
			register short* bp = blk;
			for (register int i = 6; --i >= 0; ) {
				++bp;	// ignore dc coef
				for (register int j = 63; --j >= 0; ) {
					register int v = *bp;
					*bp++ = v >> s;
				}
			}
		}
	}

	int m = mba - mba_;
	mba_ = mba;
	huffent* he = &hte_mba[m - 1];
	/* MBA */
	PUT_BITS(he->val, he->nb, nbb_, bb_);
	if (q != mquant_) {
		/* MTYPE = INTRA + TC + MQUANT */
		PUT_BITS(1, 7, nbb_, bb_);
		PUT_BITS(q, 5, nbb_, bb_);
		mquant_ = q;
	} else {
		/* MTYPE = INTRA + TC (no quantizer) */
		PUT_BITS(1, 4, nbb_, bb_);
	}

	/* luminance */
	const char* lm = llm_[q];
	if (lm == 0) {
		lm = make_level_map(q, 1);
		llm_[q] = lm;
		clm_[q] = make_level_map(q, 2);
	}
	/* luminance */
	encode_blk(blk + 0, lm);
	encode_blk(blk + 64, lm);
	encode_blk(blk + 128, lm);
	encode_blk(blk + 192, lm);
	/* chrominance */
	lm = clm_[q];
	encode_blk(blk + 256, lm);
	encode_blk(blk + 320, lm);
}

void H261DCTEncoder::encode_mb(int mba, const u_char* f, int blkno, 
			       int uvblkno, int boff, int how)
{
	/* DCT coeffs */
	const short* frm = (const short*)f;

	register int blkidx = blkno << 6;

	short ub[64];
	short vb[64];
	short* ublk;
	short* vblk;

	if (color_) {
		/*
		 * We only decimate if we need to encode the macroblock.
		 * otherwise we use the cached uv blocks.
		 */
		int fs = framesize_;

		if (decimation_ == 422) {
			ublk = ub;
			vblk = vb;
			const short* uin = &frm[fs];
			uin += blkidx >> 1;
			const short* vin = uin + (fs >> 1);
			dct_decimate(uin, uin + (boff >> 1), ublk);
			dct_decimate(vin, vin + (boff >> 1), vblk);
		} else if (decimation_ == 411) {
			ublk = (short*)&frm[fs];
			ublk += uvblkno << 6;
			vblk = ublk + (fs >> 2);
		} else {
			abort();
		}
	} else
		ublk = vblk = bwdctblk;

	register u_int q;
	if (how == CR_MOTION)
		q = lq_;
	else if (how == CR_BG)
		q = hq_;
	else
		/* must be at age threshold */
		q = mq_;

	/*
	 * If the default quantizer is too small to handle the coef.
	 * dynamic range, spin through the blocks and see if any
	 * coef. would significantly overflow.
	 */
	if (q < 8) {
		register int cmin = 0, cmax = 0;
		BLK_MINMAX(&frm[blkidx], cmin, cmax);
		BLK_MINMAX(&frm[blkidx + 64], cmin, cmax);
		BLK_MINMAX(&frm[blkidx + boff], cmin, cmax);
		BLK_MINMAX(&frm[blkidx + boff + 64], cmin, cmax);
		BLK_MINMAX(ublk, cmin, cmax);
		BLK_MINMAX(vblk, cmin, cmax);

		if (cmax < -cmin)
			cmax = -cmin;
		cmax /= (q << 1);
		if (cmax >= 128) {
			/* need to re-quantize */
			register int s;
			for (s = 1; cmax >= (128 << s); ++s)
				;
			q <<= s;
			if (q > 31)
				q >>= 1;
		}
	}

	int m = mba - mba_;
	mba_ = mba;
	huffent* he = &hte_mba[m - 1];
	/* MBA */
	PUT_BITS(he->val, he->nb, nbb_, bb_);
	if (q != mquant_) {
		/* MTYPE = INTRA + TC + MQUANT */
		PUT_BITS(1, 7, nbb_, bb_);
		PUT_BITS(q, 5, nbb_, bb_);
		mquant_ = q;
	} else {
		/* MTYPE = INTRA + TC (no quantizer) */
		PUT_BITS(1, 4, nbb_, bb_);
	}

	const char* lm = llm_[q];
	if (lm == 0) {
		/*
		 * the filter thresh is 0 since we assume the jpeg percept.
		 * quantizer already did the filtering.
		 */
		lm = make_level_map(q, 0);
		llm_[q] = lm;
		clm_[q] = make_level_map(q, 0);
	}
	/* luminance */
	encode_blk(&frm[blkidx], lm);
	encode_blk(&frm[blkidx + 64], lm);
	encode_blk(&frm[blkidx + boff], lm);
	encode_blk(&frm[blkidx + boff + 64], lm);
	/* chrominance */
	lm = clm_[q];
	encode_blk(ublk, lm);
	encode_blk(vblk, lm);
}

int H261Encoder::flush(Transmitter::pktbuf* pb, int nbit, 
		       Transmitter::pktbuf* npb, Transmitter& tx)
{
	/* flush bit buffer */
	STORE_BITS(bb_);

	int cc = (nbit + 7) >> 3;
	int ebit = (cc << 3) - nbit;

	/*XXX*/
	if (cc == 0 && npb != 0)
		abort();

	pb->iov[0].iov_len = HDRSIZE;
	pb->iov[1].iov_len = cc;
	rtphdr* rh = (rtphdr*)pb->iov[0].iov_base;
	if (npb == 0)
		rh->rh_flags |= htons(RTP_M);

	int h = *(u_int*)(rh + 1) | ebit << 26 | sbit_ << 29;
	*(u_int*)(rh + 1) = htonl(h);

	if (npb != 0) {
		u_char* nbs = (u_char*)npb->iov[1].iov_base;
		int tbit = bc_ + nbb_;
		int extra = ((tbit + 7) >> 3) - (nbit >> 3);
		memcpy(nbs, bs_ + (nbit >> 3), extra);
		bs_ = nbs;
		sbit_ = nbit & 7;
		tbit -= nbit &~ 7;
		bc_ = tbit &~ 0x1f;
		nbb_ = tbit - bc_;
		/*
		 * Prime the bit buffer.  Be careful to set bits that
		 * are not yet in use to 0, since output bits are later
		 * or'd into the buffer.
		 */
		if (nbb_ > 0) {
			bb_ = ntohl(*(u_int*)(bs_ + (bc_ >> 3)));
			bb_ &=~ ((1 << (NBIT - nbb_)) - 1);
		} else
			bb_ = 0;
	}
	tx.send(pb);

	return (cc + HDRSIZE);
}

int H261Encoder::framesize(int inw, int inh, int& outw, int& outh) const
{
	if (inw <= 176)
		outw = 176;
	else 
		outw = 352;
	
	if (inh <= 144)
		outh = 144;
	else
		outh = 288;
}

int H261Encoder::consume(const VideoFrame* vf, Transmitter& tx)
{
	if (vf->width_ != width_ || vf->height_ != height_ || 
	    vf->decimation_ != decimation_)
		size(vf->width_, vf->height_, vf->decimation_);

	return(encode(vf->ts_, vf->bp_, vf->crvec_, tx));
}

int H261Encoder::encode(n_long ts, const u_char* frm, const u_int8_t *crvec,
		        Transmitter& tx)
{
	tx.flush();

	Transmitter::pktbuf* pb = tx.alloc(ts, RTP_PT_H261);
	bs_ = (u_char*)pb->iov[1].iov_base;
	int ec = (tx.mtu() - HDRSIZE) << 3;
	bb_ = 0;
	nbb_ = 0;
	bc_ = 0;
	sbit_ = 0;
	/* RTP/H.261 header */
	rtphdr* rh = (rtphdr*)pb->iov[0].iov_base;
	*(u_int*)(rh + 1) = 1 << 25 | lq_ << 10;

	/* PSC */
	PUT_BITS(0x0001, 16, nbb_, bb_);
	/* GOB 0 -> picture header */
	PUT_BITS(0, 4, nbb_, bb_);
	/* TR (XXX should do this right) */
	PUT_BITS(0, 5, nbb_, bb_);
	/* PTYPE = CIF */
	int pt = cif_ ? 4 : 0;
	PUT_BITS(pt, 6, nbb_, bb_);
	/* PEI */
	PUT_BITS(0, 1, nbb_, bb_);

	int cc = 0;
	int step = (cif_)? 1 : 2;
	int boff = (11 << cif_) * 2 * 64;
	int blkstep = (11 << cif_) * 4;
	int uvblkstep = (decimation_ == 411)? blkstep >> 2 : blkstep >> 1;
	int gob;
	for (gob = 0; gob < ngob_; gob += step) {
		int blkno = blkno_[gob];
		int uvblkno = uvblkno_[gob];
		int nbit = bc_ + nbb_;

		/* GSC/GN */
		PUT_BITS(0x10 | (gob + 1), 20, nbb_, bb_);
		/* GQUANT/GEI */
		mquant_ = lq_;
		PUT_BITS(mquant_ << 1, 6, nbb_, bb_);

		mba_ = 0;
		int line = 11;
		for (int mba = 1; mba <= 33; ++mba) {
			/*
			 * If the conditional replenishment algorithm
			 * has decided to send any of the blocks of
			 * this macroblock, code it.
			 */
			int s = crvec[blkno];
			s |= crvec[blkno + 1];
			s |= crvec[blkno + (blkstep >> 1)];
			s |= crvec[blkno + (blkstep >> 1) + 1];
			if ((s & CR_SEND) != 0) {
				int mbpred = mba_;
				encode_mb(mba, frm, blkno, uvblkno, boff, 
					  CR_STATE(s));
				if (bc_ + nbb_ > ec) {
					Transmitter::pktbuf* npb;
					npb = tx.alloc(ts, RTP_PT_H261);
					cc += flush(pb, nbit, npb, tx);
					pb = npb;
					/* RTP/H.261 header */
					int m = mbpred;
					int g;
					if (m != 0) {
						g = gob + 1;
						m -= 1;
					} else
						g = 0;

					rh = (rtphdr*)pb->iov[0].iov_base;
					*(u_int*)(rh + 1) =
						1 << 25 |
						m << 15 |
						g << 20 |
						mquant_ << 10;
				}
				nbit = bc_ + nbb_;
			}
			blkno += 2; 
			uvblkno += 1;
			if (--line <= 0) {
				line = 11;
				blkno -= 11 * 2;
				blkno += blkstep;
				uvblkno -= 11;
				uvblkno += uvblkstep;
			}
		}
	}
	cc += flush(pb, bc_ + nbb_, 0, tx);
	return (cc);
}
